# at coord we put symbol
# 0-0
import itertools
import math

from termcharts.colors import Color
from termcharts.colors import default_colors
from termcharts.engine import add_char
from termcharts.engine import coord_in_scr
from termcharts.engine import coord_to_str
from termcharts.engine import get_coord
from termcharts.engine import pie_add_text as add_text
from termcharts.engine import pie_render as render


def corx(xc, r, theta):
    return xc + (r * math.cos(math.radians(theta)))


def cory(yc, r, theta):
    return yc + (r * math.sin(math.radians(theta)))


grid_s = 20
centerx = 250
centery = 250
rx = 200
ry = 200


def sector_line(centerx, centery, rx, ry, degstart, degend, grid_s, screen, symbol):
    for i in range(degstart, degend):
        x = corx(centerx, rx, i)
        y = cory(centery, ry, i)

        gx = int(x // grid_s)
        gy = int(y // grid_s)

        add_char(screen, [gx, gy], symbol)


def pie_chart_raw(
    data: dict,
    centerx,
    centery,
    rx,
    ry,
    grid_s,
    screen,
    return_rich=False,
    __debug=False,
    fill=False,
    title="pie chart",
):
    total = sum(data[d] for d in data)
    deg_current = 0
    # for i in range(10):

    if fill == True:
        fill_factor = 21
    else:
        fill_factor = 5

    max_d_len = sum(len(e) for e in data)
    for s, d in enumerate(data):

        deg_step = int((data[d] / total) * 360)
        col = next(default_colors)

        # legend
        # add_text(screen, col + d, 25, s + 2)
        # add_text(screen, 'x', 25, 3)
        # add_char(screen, [10, 3], '#')

        # print(col+d)
        # print(col+d)
        if not return_rich:
            txt = f"{col} {d} {Color.RESET}"
        else:
            txt = f"{col} {d}"

        # print(d, col+d, col+d+Color.RESET, file=open('z.log', 'w+'))

        # txt = d
        add_text(screen, txt, 25, s + 2)

        for i in range(10):

            if not return_rich:
                text = col + "█" + Color.RESET  # reset col
            else:
                text = col + "█"
            sector_line(
                centerx,
                centery,
                rx - (i * fill_factor),
                ry - (i * fill_factor),
                deg_current,
                deg_current + deg_step,
                grid_s,
                screen,
                text,
            )
        deg_current += deg_step

    add_text(screen, title, 0, 1)

    display = 50 + max_d_len + 3

    source = render(screen, display - (25), display - (9 + max_d_len), debug=__debug)

    if return_rich:
        try:
            from rich.text import Text
        except ImportError:
            raise Exception("Text not found from rich.text")
        return Text.from_ansi(source)
    else:
        return source


def pie_chart(data, rich=False, title="pie chart", screen=None):
    global centerx, centery, rx, ry, grid_s
    fill = True
    if screen is None:
        screen = {}
    return pie_chart_raw(
        data,
        centerx,
        centery,
        rx,
        ry,
        grid_s,
        screen,
        fill=fill,
        return_rich=rich,
        title=title,
    )


def doughnut_chart(data, rich=False, title="doughnut chart", screen=None):
    global centerx, centery, rx, ry, grid_s
    fill = False
    if screen is None:
        screen = {}
    return pie_chart_raw(
        data,
        centerx,
        centery,
        rx,
        ry,
        grid_s,
        screen,
        fill=fill,
        return_rich=rich,
        title=title,
    )



import itertools

# if need instantiation use dataclass
class Color:
    RESET = "\033[39m"

    red = "\033[31m"
    green = "\033[32m"
    orange = "\033[33m"
    purple = "\033[35m"
    white = "\033[37m"


default_colors = itertools.cycle([Color.red, Color.green, Color.orange, Color.purple])



from collections import ChainMap
from typing import Union


def coord_to_str(coord):
    r = "-".join([str(i) for i in coord])
    return r


def add_char(screen_: dict, coord: Union[list, tuple], value):
    # add coord to screen
    coord_str = coord_to_str(coord)

    screen_[coord_str] = value


def get_coord(screen, coord: Union[list, tuple]):
    return screen[coord_to_str(coord)]


def coord_in_scr(screen: dict, coord: Union[list, tuple]):
    return coord_to_str(coord) in screen


def pie_render(screen, displayx, displayy, debug=False):
    displayx, displayy = displayy, displayx

    c = 0
    xxx = ""
    for x in range(displayx - 20):
        for y in range(displayy):
            c += 1
            if c % (displayy) == 0:
                if debug:
                    end = "|\n"
                else:
                    end = "\n"
            else:
                end = ""

            if x <= displayx and y <= displayy:
                # print(x, y, displayx, displayy)
                if not coord_in_scr(screen, [x, y]):
                    xxx = xxx + " " + end
                else:
                    xxx = xxx + get_coord(screen, [x, y]) + end

    return xxx


def render(screen, size_x, size_y):
    RESET = "\033[39m"
    string = ""
    end = ""
    counter = 0
    for y in range(size_y):
        for x in range(size_x):
            counter += 1
            if counter % size_x == 0:
                end = "\n"
            else:
                end = ""

            coord = f"{x}-{y}"

            if x <= size_x and y <= size_y:
                if not coord_in_scr(screen, [x, y]):
                    string = string + " " + end
                else:
                    string = string + get_coord(screen, [x, y]) + end

    return string


def pie_add_text(screen_, text, gx, gy, mode="h"):
    x = 0
    y = 0

    for c in text:
        if "h" in mode:
            add_char(screen_, [gy, gx + x], c)
            x += 1
        if "v" in mode:
            add_char(screen_, [gy + y, x], c)
            y += 1


def add_text(screen_, text: Union[list, str], gx, gy, mode="h"):
    """List is the preferred text type, colors integrate without problem"""
    x = 0
    y = 0

    for c in text:
        if "h" in mode:
            add_char(screen_, [gx + x, gy], c)
            x += 1
        if "v" in mode:
            add_char(screen_, [gx, gy + y], c)
            y += 1


def screen():
    return {}


def merge_screens(screens):
    screens = reversed(screens)
    return dict(ChainMap(*screens))



def constrain(
    val, start, end, realstart, realend, lessthan_sym=None, morethan_sym=None
):
    # from hooman
    # mouseX 0 width 0 255
    # v = (mouseX / (end-start)) * (realend-realstart)
    # return realstart + v
    # if val < start, val = start
    # if val > end, val = end

    if val < start:
        if lessthan_sym is None:
            return start
        else:
            return lessthan_sym
    if val > end:
        if morethan_sym is None:
            return end
        else:
            return morethan_sym
    v = ((val - start) / (end - start)) * (realend - realstart)
    return realstart + v



import math
from itertools import cycle

from termcharts.colors import Color
from termcharts.colors import default_colors
from termcharts.engine import add_char
from termcharts.engine import add_text
from termcharts.engine import merge_screens
from termcharts.engine import render
from termcharts.engine import screen
from termcharts.formula import constrain


def bar_chart_raw_h(data, title, return_rich=False):
    top_pad = 3
    right_pad = 2
    values_x = 500
    values_y = 100

    term_size_x = 40 + 1
    term_size_y = top_pad + len(data) + (len(data) - 1) + 2

    screen_axis = screen()
    screen_chart = screen()

    # Add left bar
    for i in range(term_size_x):
        add_char(screen_axis, [0, i + top_pad], Color.white + "│" + Color.RESET)

    max_item = max(data[e] for e in data)
    max_d_len = max(len(e) for e in data)

    # Draw bars
    space_pad = 1
    for i, item in enumerate(data):
        col = next(default_colors)
        width = term_size_x - (right_pad + max_d_len + 5)
        number = int(constrain(data[item], 0, max_item, 0, width))
        bar = f"{col}" + ("█" * number)

        if not return_rich:
            bar = bar + f" {data[item]}" + Color.RESET
        else:
            bar = bar + f" {data[item]}"
        add_text(screen_chart, bar, 1, top_pad + i + space_pad)
        space_pad += 1

    # Add title

    add_text(screen_chart, Color.white + title, 0, 1)

    main_screen = merge_screens([screen_chart, screen_axis])

    source = render(main_screen, term_size_x, term_size_y)

    if return_rich:
        try:
            from rich.text import Text
        except ImportError:
            raise Exception("Text not found from rich.text")
        return Text.from_ansi(source)
    else:
        return source


def bar_chart_raw_v(data, title, return_rich=False):
    top_pad = 4
    right_pad = 2
    values_x = 500
    values_y = 100
    bottom_pad = 1

    max_item = max(data[e] for e in data)
    max_d_len = max(len(e) for e in data)

    term_size_y = 15
    chart_size_x = len(data) + (len(data) - 1) + 2
    if title is None:
        term_size_x = chart_size_x
    elif title:
        if len(title) > chart_size_x:
            term_size_x = len(title) + 2
        else:
            term_size_x = chart_size_x + 5

    screen_axis = screen()
    screen_chart = screen()

    # Draw bars
    space_pad = 1

    for i, item in enumerate(data):
        print(item)
        col = next(default_colors)
        height = term_size_y - (top_pad + bottom_pad)
        number = int(constrain(data[item], 0, max_item, 0, height))

        if not return_rich:
            bar = [f"{col}█{Color.RESET}" for i in range(number + 1)]
        else:
            bar = [f"{col}█" for i in range(number + 1)]
        txt_start_y = term_size_y - len(bar) - bottom_pad
        add_text(screen_chart, bar, i + space_pad, txt_start_y, mode="v")
        space_pad += 1

    # Add title
    add_text(screen_chart, Color.white + title, 0, 1)

    # Add bottom bar
    add_text(screen_axis, [f"{Color.white}─" * chart_size_x], 0, term_size_y - 1)

    # ---
    main_screen = merge_screens([screen_chart, screen_axis])

    source = render(main_screen, term_size_x, term_size_y)

    if return_rich:
        try:
            from rich.text import Text
        except ImportError:
            raise Exception("Text not found from rich.text")
        return Text.from_ansi(source)
    else:
        return source


def bar(data, title, rich=False, mode="h"):
    if mode == "h":
        return bar_chart_raw_h(data, title, return_rich=rich)
    if mode == "v":
        return bar_chart_raw_v(data, title, return_rich=rich)
